@@ -1,19 +1,20 @@
 package Workflow::Condition::LazyOR;
 
-use strict;
 use warnings;
+use strict;
+use v5.14.0;
 
-our $VERSION = '1.62';
+our $VERSION = '2.02';
 
-use base qw( Workflow::Condition::Nested );
-use Workflow::Exception qw( condition_error configuration_error );
-use English qw( -no_match_vars );
+use parent qw( Workflow::Condition );
+use Workflow::Exception qw( configuration_error );
 
 __PACKAGE__->mk_accessors('conditions');
 
 
-sub _init {
+sub init {
     my ( $self, $params ) = @_;
+    $self->SUPER::init( $params );
 
     # This is a tricky one. The admin may have configured this by repeating
     # the param name "condition" or by using unique names (e.g.: "condition1",
@@ -25,7 +26,6 @@ sub _init {
         push @conditions, $self->normalize_array( $params->{$key} );
     }
     $self->conditions( [@conditions] );
-
 }
 
 sub evaluate {
@@ -37,10 +37,10 @@ sub evaluate {
     foreach my $cond ( @{$conditions} ) {
         my $result = $self->evaluate_condition( $wf, $cond );
         if ($result) {
-            return $result;
+            return Workflow::Condition::IsTrue->new("Got match in " . $cond );
         }
     }
-    condition_error("All nested conditions returned 'false'");
+    return Workflow::Condition::IsFalse->new();
 }
 
 1;
@@ -55,7 +55,7 @@ Workflow::Condition::LazyOR
 
 =head1 VERSION
 
-This documentation describes version 1.62 of this package
+This documentation describes version 2.02 of this package
 
 =head1 DESCRIPTION
 
@@ -111,7 +111,7 @@ B<or>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003-2023 Chris Winters. All rights reserved.
+Copyright (c) 2003-2021 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
