@@ -2,14 +2,11 @@ package Workflow::Exception;
 
 use warnings;
 use strict;
+use v5.14.0;
 
 # Declare some of our exceptions...
 
 use Exception::Class (
-    'Workflow::Exception::Condition' => {
-        isa         => 'Workflow::Exception',
-        description => 'Condition failed errors',
-    },
     'Workflow::Exception::Configuration' => {
         isa         => 'Workflow::Exception',
         description => 'Configuration errors',
@@ -25,26 +22,23 @@ use Exception::Class (
     },
 );
 
-use Log::Log4perl qw( get_logger );
-use Log::Log4perl::Level;
+use Log::Any;
 
 my %TYPE_CLASSES = (
-    condition_error     => 'Workflow::Exception::Condition',
     configuration_error => 'Workflow::Exception::Configuration',
     persist_error       => 'Workflow::Exception::Persist',
     validation_error    => 'Workflow::Exception::Validation',
     workflow_error      => 'Workflow::Exception',
 );
 my %TYPE_LOGGING = (
-    condition_error     => $TRACE,
-    configuration_error => $ERROR,
-    persist_error       => $ERROR,
-    validation_error    => $INFO,
-    workflow_error      => $ERROR,
+    configuration_error => 'error',
+    persist_error       => 'error',
+    validation_error    => 'info',
+    workflow_error      => 'error',
 );
 
 
-$Workflow::Exception::VERSION   = '1.62';
+$Workflow::Exception::VERSION   = '2.02';
 @Workflow::Exception::ISA       = qw( Exporter Exception::Class::Base );
 @Workflow::Exception::EXPORT_OK = keys %TYPE_CLASSES;
 
@@ -55,14 +49,16 @@ sub _mythrow {
 
     my ( $msg, %params ) = _massage(@items);
     my $caller = caller;
-    my $log = get_logger($caller); # log as if part of the package of the caller
+    my $log = Log::Any->get_logger( category => $caller ); # log as if part of the package of the caller
     my ( $pkg, $line ) = (caller)[ 0, 2 ];
     my ( $prev_pkg, $prev_line ) = ( caller 1 )[ 0, 2 ];
 
     # Do not log condition errors
-    $log->log( $TYPE_LOGGING{$type},
-               "$type exception thrown from [$pkg: $line; before: ",
-               "$prev_pkg: $prev_line]: $msg" );
+    my $method = $TYPE_LOGGING{$type};
+    $log->$method(
+        "$type exception thrown from [$pkg: $line; before: ",
+        "$prev_pkg: $prev_line]: $msg"
+    );
 
     goto &Exception::Class::Base::throw(
         $TYPE_CLASSES{$type},
@@ -73,11 +69,6 @@ sub _mythrow {
 
 # Use 'goto' here to maintain the stack trace
 
-sub condition_error {
-    unshift @_, 'condition_error';
-    goto &_mythrow;
-}
-
 sub configuration_error {
     unshift @_, 'configuration_error';
     goto &_mythrow;
@@ -129,7 +120,7 @@ Workflow::Exception - Base class for workflow exceptions
 
 =head1 VERSION
 
-This documentation describes version 1.62 of this package
+This documentation describes version 2.02 of this package
 
 =head1 SYNOPSIS
 
@@ -190,12 +181,6 @@ makes for very readable code:
                 "frightfully wrong: $@",
                 { foo => 'bar' };
 
-=head3 condition_error
-
-This method transforms the error to a condition error.
-
-This exception is thrown via </mythrow> when a condition of a workflow is invalid.
-
 =head3 configuration_error
 
 This method transforms the error to a configuration error.
@@ -226,8 +211,6 @@ This exception is thrown via </mythrow> when input data or similar of a workflow
 
 =item * B<Workflow::Exception> - import using C<workflow_error>
 
-=item * B<Workflow::Exception::Condition> - import using C<condition_error>
-
 =item * B<Workflow::Exception::Configuration> - import using C<configuration_error>
 
 =item * B<Workflow::Exception::Persist> - import using C<persist_error>
@@ -246,7 +229,7 @@ This exception is thrown via </mythrow> when input data or similar of a workflow
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003-2023 Chris Winters. All rights reserved.
+Copyright (c) 2003-2021 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
