@@ -2,10 +2,11 @@ package Workflow::History;
 
 use warnings;
 use strict;
-use base qw( Class::Accessor );
+use v5.14.0;
+use parent qw( Class::Accessor );
 use DateTime;
 
-$Workflow::History::VERSION = '1.62';
+$Workflow::History::VERSION = '2.02';
 
 my @FIELDS
     = qw( id workflow_id action description date user state time_zone );
@@ -66,7 +67,7 @@ Workflow::History - Recorded work on a workflow action or workflow itself
 
 =head1 VERSION
 
-This documentation describes version 1.62 of this package
+This documentation describes version 2.02 of this package
 
 =head1 SYNOPSIS
 
@@ -75,9 +76,12 @@ This documentation describes version 1.62 of this package
      my ( $self, $wf ) = @_;
      my $current_user = $wf->context->param( 'current_user' );
      # ... do your work with $ticket
-     $wf->add_history( action => 'create ticket',
-                       user   => $current_user->full_name,
-                       description => "Ticket $ticket->{subject} successfully created" );
+     $wf->add_history(
+         {
+            action => 'create ticket',
+            user   => $current_user->full_name,
+            description => "Ticket $ticket->{subject} successfully created"
+         });
  }
 
  # in your view (using TT2)
@@ -107,17 +111,18 @@ C<\%params>.
 =head3 set_new_state( $new_state )
 
 Assigns the new state C<$new_state> to the history if the state is not
-already assigned. This is used when you generate a history request in
+already assigned. This is used when you
+L<generate a history request|Workflow/add_history> in
 a L<Workflow::Action> since the workflow state will change once the
-action has successfully completed. So in the action you create a
-history object without the state:
+action has successfully completed. So in the action you create
+history without the state:
 
   $wf->add_history(
-      Workflow::History->new({
+      {
           action      => "Cocoa Puffs",
           description => "They're magically delicious",
           user        => "Count Chocula",
-      })
+      }
   );
 
 And then after the new state has been set but before the history
@@ -185,7 +190,7 @@ Sets saved state to true and returns 1
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003-2023 Chris Winters. All rights reserved.
+Copyright (c) 2003-2021 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
