@@ -2,13 +2,14 @@ package Workflow::Persister::DBI::SequenceId;
 
 use warnings;
 use strict;
-use base qw( Class::Accessor );
+use v5.14.0;
+use parent qw( Class::Accessor );
 use DBI;
-use Log::Log4perl qw( get_logger );
+use Log::Any;
 use Workflow::Exception qw( persist_error );
-use English qw( -no_match_vars );
+use Syntax::Keyword::Try;
 
-$Workflow::Persister::DBI::SequenceId::VERSION = '1.62';
+$Workflow::Persister::DBI::SequenceId::VERSION = '2.02';
 
 my @FIELDS = qw( log sequence_name sequence_select );
 __PACKAGE__->mk_accessors(@FIELDS);
@@ -17,9 +18,9 @@ __PACKAGE__->mk_accessors(@FIELDS);
 sub new {
     my ( $class, $params ) = @_;
     $params ||= {};
-    $params->{log} = get_logger( $class );
+    $params->{log} = Log::Any->get_logger( category => $class );
 
-    return bless { %$params }, $class;
+    return bless { %{$params} }, $class;
 }
 
 sub pre_fetch_id {
@@ -27,16 +28,14 @@ sub pre_fetch_id {
     my $full_select = sprintf $self->sequence_select, $self->sequence_name;
     $self->log->debug("SQL to fetch sequence: ", $full_select);
     my ($row);
-
-    local $EVAL_ERROR = undef;
-    eval {
+    try {
         my $sth = $dbh->prepare($full_select);
         $sth->execute;
         $row = $sth->fetchrow_arrayref;
         $sth->finish;
-    };
-    if ($EVAL_ERROR) {
-        persist_error "Failed to retrieve sequence: $EVAL_ERROR";
+    }
+    catch ($error) {
+        persist_error "Failed to retrieve sequence: $error";
     }
     return $row->[0];
 }
@@ -55,7 +54,7 @@ Workflow::Persister::DBI::SequenceId - Persister to fetch ID from a sequence
 
 =head1 VERSION
 
-This documentation describes version 1.62 of this package
+This documentation describes version 2.02 of this package
 
 =head1 SYNOPSIS
 
@@ -114,7 +113,7 @@ This is a I<dummy> method, use L</pre_fetch_id>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003-2023 Chris Winters. All rights reserved.
+Copyright (c) 2003-2021 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
