@@ -2,16 +2,17 @@ package Workflow::Persister::File;
 
 use warnings;
 use strict;
-use base qw( Workflow::Persister );
+use v5.14.0;
+use parent qw( Workflow::Persister );
 use Data::Dumper qw( Dumper );
+use English qw( -no_match_vars );
 use File::Spec::Functions qw( catdir catfile );
-use Log::Log4perl qw( get_logger );
 use Workflow::Exception qw( configuration_error persist_error );
 use Workflow::Persister::RandomId;
 use File::Slurp qw(slurp);
-use English qw( -no_match_vars );
+use Syntax::Keyword::Try;
 
-$Workflow::Persister::File::VERSION = '1.62';
+$Workflow::Persister::File::VERSION = '2.02';
 
 my @FIELDS = qw( path );
 __PACKAGE__->mk_accessors(@FIELDS);
@@ -61,12 +62,13 @@ sub fetch_workflow {
         persist_error "No workflow with ID '$wf_id' is available";
     }
     $self->log->debug("File exists, reconstituting workflow");
-
-    local $EVAL_ERROR = undef;
-    my $wf_info = eval { $self->constitute_object($full_path) };
-    if ($EVAL_ERROR) {
+    my $wf_info;
+    try {
+        $wf_info = $self->constitute_object($full_path);
+    }
+    catch ($error) {
         persist_error "Cannot reconstitute data from file for ",
-            "workflow '$wf_id': $EVAL_ERROR";
+            "workflow '$wf_id': $error";
     }
     return $wf_info;
 }
@@ -90,7 +92,9 @@ sub create_history {
         my $history_id = $generator->pre_fetch_id();
         $history->id($history_id);
         my $history_file = catfile( $history_dir, $history_id );
-        $self->serialize_object( $history_file, $history );
+        # Serialize as hash so reconstituting the object returns a hash again
+        # we need to return a list of hashes when returning the history list.
+        $self->serialize_object( $history_file, { %$history } );
         $self->log->info("Created history object '$history_id' ok");
         $history->set_saved();
     }
@@ -110,7 +114,6 @@ sub fetch_history {
     foreach my $history_file (@history_files) {
         $self->log->debug("Reading history from file '$history_file'");
         my $history = $self->constitute_object($history_file);
-        $history->set_saved();
         push @histories, $history;
     }
     return @histories;
@@ -126,8 +129,7 @@ sub _serialize_workflow {
         state       => $wf->state,
         last_update => $wf->last_update,
         type        => $wf->type,
-        context     => $wf->context,
-
+        context     => { %{$wf->context->{PARAMS} } },
     );
     $self->serialize_object( $full_path, \%wf_info );
     $self->log->debug("Wrote workflow ok");
@@ -151,11 +153,18 @@ sub constitute_object {
     my $content = slurp($object_path);
 
     no strict;
-    local $EVAL_ERROR = undef;
-    my $object = eval $content;
-    croak $EVAL_ERROR if ($EVAL_ERROR);
+    my $object;
+    my $error;
+    my $success = do {
+        local $@;
+        my $rv = eval "\$object = do { $content }; 1;";
+        $error = $EVAL_ERROR;
+        $rv;
+    };
+    if (not $success) {
+        die $error;
+    }
     return $object;
-
 }
 
 sub _get_workflow_path {
@@ -182,7 +191,7 @@ Workflow::Persister::File - Persist workflow and history to the filesystem
 
 =head1 VERSION
 
-This documentation describes version 1.62 of this package
+This documentation describes version 2.02 of this package
 
 =head1 SYNOPSIS
 
@@ -294,7 +303,7 @@ to deserialization attempt.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003-2023 Chris Winters. All rights reserved.
+Copyright (c) 2003-2021 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
