@@ -2,13 +2,14 @@ package Workflow::Persister::RandomId;
 
 use warnings;
 use strict;
-use base qw( Class::Accessor );
+use v5.14.0;
+use parent qw( Class::Accessor );
 
 use constant DEFAULT_ID_LENGTH  => 8;
 use constant RANDOM_SEED        => 26;
 use constant CONSTANT_INCREMENT => 65;
 
-$Workflow::Persister::RandomId::VERSION = '1.62';
+$Workflow::Persister::RandomId::VERSION = '2.02';
 
 my @FIELDS = qw( id_length );
 __PACKAGE__->mk_accessors(@FIELDS);
@@ -42,7 +43,7 @@ Workflow::Persister::RandomId - Persister to generate random ID
 
 =head1 VERSION
 
-This documentation describes version 1.62 of this package
+This documentation describes version 2.02 of this package
 
 =head1 SYNOPSIS
 
@@ -84,7 +85,7 @@ This method is unimplemented at this time, please see the TODO.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003-2023 Chris Winters. All rights reserved.
+Copyright (c) 2003-2021 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
