@@ -2,16 +2,15 @@ package Workflow::State;
 
 use warnings;
 use strict;
-use base qw( Workflow::Base );
-use Log::Log4perl qw( get_logger );
+use v5.14.0;
+use parent qw( Workflow::Base );
 use Workflow::Condition;
 use Workflow::Condition::Evaluate;
-use Workflow::Exception qw( workflow_error condition_error );
+use Workflow::Exception qw( workflow_error );
 use Exception::Class;
 use Workflow::Factory qw( FACTORY );
-use English qw( -no_match_vars );
 
-$Workflow::State::VERSION = '1.62';
+$Workflow::State::VERSION = '2.02';
 
 my @FIELDS   = qw( state description type );
 my @INTERNAL = qw( _test_condition_count _factory _actions _conditions
@@ -78,18 +77,7 @@ sub get_available_action_names {
 
 sub is_action_available {
     my ( $self, $wf, $action_name ) = @_;
-    local $EVAL_ERROR = undef;
-    eval { $self->evaluate_action( $wf, $action_name ) };
-
-    # Everything is fine
-    return 1 unless( $EVAL_ERROR );
-
-    # We got an exception, check if it is a Workflow::Exception
-    return 0 if (Exception::Class->caught('Workflow::Exception'));
-
-    $EVAL_ERROR->rethrow() if (ref $EVAL_ERROR);
-
-    croak $EVAL_ERROR;
+    return $self->evaluate_action( $wf, $action_name );
 }
 
 sub clear_condition_cache {
@@ -107,32 +95,19 @@ sub evaluate_action {
     my @conditions = $self->get_conditions($action_name);
     foreach my $condition (@conditions) {
         my $condition_name = $condition->name;
+        my $rv = Workflow::Condition->evaluate_condition($wf, $condition_name);
+        if (! $rv) {
 
-        my $rv;
-        local $EVAL_ERROR = undef;
-        eval {
-            $rv = Workflow::Condition->evaluate_condition($wf, $condition_name);
-        };
-        if ($EVAL_ERROR) {
-            if (Exception::Class->caught('Workflow::Exception::Condition')) {
-                condition_error "No access to action '$action_name' in ",
-                    "state '$state' because $EVAL_ERROR ";
-            }
-            else {
-                $EVAL_ERROR->rethrow() if (ref $EVAL_ERROR ne '');
-                # For briefness, we just send back the first line of EVAL
-                my @t = split /\n/, $EVAL_ERROR;
-                my $ee = shift @t;
-                Exception::Class::Base->throw(
-                    error
-                    => "Got unknown exception while handling condition '$condition_name' / " . $ee );
-            }
-        }
-        elsif (! $rv) {
-            condition_error "No access to action '$action_name' in ",
-                "state '$state' because condition '$condition_name' failed";
+            $self->log->is_debug
+                && $self->log->debug(
+                "No access to action '$action_name' in ",
+                "state '$state' because condition '$condition_name' failed");
+
+            return $rv;
         }
     }
+
+    return 1;
 }
 
 sub get_next_state {
@@ -141,13 +116,20 @@ sub get_next_state {
     my $resulting_state = $self->_next_state->{$action_name};
     return $resulting_state unless ( ref($resulting_state) eq 'HASH' );
 
-    if ( defined $action_return ) {
+    return %{$resulting_state} unless(defined $action_return);
+
+    my $state = $self->state;
+    workflow_error "State->get_next_state was called with a non-scalar ",
+        "return value in state '$state' on action '$action_name'" if (ref $action_return ne '');
+
+    return $resulting_state->{$action_return} if ($resulting_state->{$action_return});
+
+    return $resulting_state->{'*'} if ($resulting_state->{'*'});
+
+    workflow_error "State '$state' does not define a next state ",
+        "for a return value of '$action_return' and there is ",
+        "also no default state set.";
 
-       # TODO: Throw exception if $action_return not found and no '*' defined?
-        return $resulting_state->{$action_return} || $resulting_state->{'*'};
-    } else {
-        return %{$resulting_state};
-    }
 }
 
 sub get_autorun_action_name {
@@ -355,13 +337,14 @@ Workflow::State - Information about an individual state in a workflow
 
 =head1 VERSION
 
-This documentation describes version 1.62 of this package
+This documentation describes version 2.02 of this package
 
 =head1 SYNOPSIS
 
  # This is an internal object...
  <workflow...>
    <state name="Start">
+     <description>My state documentation</description> <!-- optional -->
      <action ... resulting_state="Progress" />
    </state>
       ...
@@ -412,34 +395,58 @@ returns. For instance we might have something like:
          <resulting_state return="helpdesk" state="Assign as Helpdesk" />
          <resulting_state return="*"        state="Assign as Luser" />
      </action>
-   </state>
+  </state>
 
 So if we execute 'create' the workflow will be in one of three states:
 'Assign as Admin' if the return value of the 'create' action is
 'admin', 'Assign as Helpdesk' if the return is 'helpdesk', and 'Assign
 as Luser' if the return is anything else.
 
+=head2 Action availability
+
+A state can have multiple actions associated with it, demonstrated in the
+first example under L</Resulting State>. The set of I<available> actions is
+a subset of all I<associated> actions: those actions for which none of the
+associated conditions fail their check.
+
+  <state name="create user">
+     <action name="create">
+         ... (resulting_states) ...
+         <condition name="can_create_users" />
+     </action>
+  </state>
+
+
+
 =head2 Autorun State
 
 You can also indicate that the state should be automatically executed
 when the workflow enters it using the 'autorun' property. Note the
 slight change in terminology -- typically we talk about executing an
 action, not a state. But we can use both here because an automatically
-run state requires that one and only one action is available for
+run state requires that one and only one action is I<available> for
 running. That doesn't mean a state contains only one action. It just
-means that only one action is available when the state is entered. For
+means that only one action is I<available> when the state is entered. For
 example, you might have two actions with mutually exclusive conditions
 within the autorun state.
 
-If no action or more than one action is available at the time the
-workflow enters an autorun state, Workflow will throw an error. There
-are some conditions where this might not be what you want. For example
-when you have a state which contains an action that depends on some
-condition. If it is true, you might be happy to move on to the next
-state, but if it is not, you are fine to come back and try again later
-if the action is available. This behaviour can be achived by setting the
-'may_stop' property to yes, which will cause Workflow to just quietly
-stop automatic execution if it does not have a single action to execute.
+=head3 Stoppable autorun states
+
+If no action or more than one action is I<available> at the time the
+workflow enters an autorun state, Workflow can't continue execution.
+If this is isn't a problem, a state may be marked with C<may_stop="yes">:
+
+
+   <state name="Approved" autorun="yes" may_stop="yes">
+     <action name="Archive" resulting_state="Completed" />
+        <condition name="allowed_automatic_archival" />
+     </action>
+  </state>
+
+
+However, in case the state isn't marked C<may_stop="yes">, Workflow will
+throw a C<workflow_error> indicating an autorun problem.
+
 
 =head1 PUBLIC METHODS
 
@@ -505,7 +512,7 @@ Returns name of action to be used for autorunning the state.
 
 =head3 clear_condition_cache ( )
 
-Deprecated, kept for 1.62 compatibility.
+Deprecated, kept for 2.02 compatibility.
 
 Used to empties the condition result cache for a given state.
 
@@ -557,7 +564,7 @@ performing some sanity checks like ensuring every action has a
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003-2023 Chris Winters. All rights reserved.
+Copyright (c) 2003-2021 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
