@@ -1,14 +1,14 @@
 package TestApp::Action::TicketUpdate;
 
 use strict;
-use base qw( Workflow::Action );
-use Log::Log4perl qw( get_logger );
+use parent qw( Workflow::Action );
+use Log::Any qw( $log );
 
 $TestApp::Action::TicketUpdate::VERSION = '1.05';
 
 sub execute {
     my ( $self, $wf ) = @_;
-    my $log = get_logger();
+
     $log->debug( "Action '", $self->name, "' with class '", ref( $self ), "' executing..." );
     my $ticket = $wf->context->param( 'ticket' );
     $ticket->status( $wf->state );
